@@ -29,6 +29,10 @@ Represents a slot in the context object.
 typedef ptrdiff_t im_slot_t;
 typedef void (*im_slot_destroy_t)(void *);
 
+/* just so we can use our own input typemap */
+typedef double im_double;
+typedef float im_float;
+
 /* used for palette indices in some internal code (which might be 
    exposed at some point
 */
@@ -1075,6 +1079,14 @@ typedef struct {
 
 typedef struct i_render_tag i_render;
 
+/* trim colors for im_trim()/im_trim_dim() */
+
+typedef struct {
+  int is_float;
+  i_color c1, c2;
+  i_fcolor cf1, cf2;
+} i_trim_colors_t;
+
 /*
 =item i_color_model_t
 =category Data Types
